import {Injectable} from '@angular/core';
import {ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot} from '@angular/router';
import {Observable} from 'rxjs';
import {Session} from './utils/helper.class';
import {LoginErrorCodes} from './utils/guardHelper.class';
import {filter, map} from "rxjs/operators";
import {UserManagementService} from "../services/user-management.service";

@Injectable()
export class ClaimsCuratorGuard implements CanActivate {
  
  constructor(private router: Router,
              private userManagementService: UserManagementService) {
  }
  
  check(path: string): Observable<boolean> | boolean {
    let errorCode = LoginErrorCodes.NOT_LOGIN;
    const obs = this.userManagementService.getUserInfo(false).pipe(map(user => {
      if (user) {
        errorCode = LoginErrorCodes.NOT_ADMIN;
      }
      return Session.isClaimsCurator(user) || Session.isPortalAdministrator(user);
    }));
    obs.pipe(filter(isAdmin => !isAdmin)).subscribe(() => {
      this.router.navigate(['/user-info'], {
        queryParams: {
          'errorCode': errorCode,
          'redirectUrl': path
        }
      });
    });
    return obs;
  }
  
  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.check(state.url);
  }
  
}
