import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {AdvancedField} from './searchUtils/searchHelperClasses.class';
import {SearchProjectsService} from '../services/searchProjects.service';
import {ErrorCodes} from '../utils/properties/errorCodes';
import {ErrorMessagesComponent} from '../utils/errorMessages.component';
import {SearchFields} from '../utils/properties/searchFields';
import {SearchCustomFilter, SearchUtilsClass} from './searchUtils/searchUtils.class';
import {EnvProperties} from '../utils/properties/env-properties';
import {NewSearchPageComponent} from "./searchUtils/newSearchPage.component";
import {RangeFilter} from "../utils/rangeFilter/rangeFilterHelperClasses.class";

@Component({
    selector: 'search-projects',
    template: `
      <new-search-page
        pageTitle="{{(simpleView?'':'Advanced ')}} Search for {{ 'projects' | titlecase }}"
        entityType="project"
        type="projects"
        [results]="results"
        [searchUtils]="searchUtils"
        [fieldIds]="fieldIds" [fieldIdsMap]="fieldIdsMap" [selectedFields]="selectedFields"
        [csvParams]="csvParams" csvPath="projects"
        [simpleSearchLink]="simpleSearchLink" [advancedSearchLink]="advancedSearchLink"
        [disableForms]="disableForms"
        [loadPaging]="loadPaging"
        [oldTotalResults]="oldTotalResults"
        [openaireLink]=openaireLink
        [piwikSiteId]=piwikSiteId [hasPrefix]="hasPrefix"
        searchFormClass="projectsSearchForm"
        [includeOnlyResultsAndFilter]="includeOnlyResultsAndFilter"
        [filters]="filters"
        [rangeFilters]="rangeFilters" [rangeFields]="rangeFields"
        [simpleView]="simpleView" formPlaceholderText="Search by title, acronym, project code..."
      >
      </new-search-page>

    `
 })

export class SearchProjectsComponent {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
@Input() piwikSiteId = null;
@Input() customFilter:SearchCustomFilter= null;
  public results =[];
  public filters =[];
  public rangeFilters: RangeFilter[] = [];

  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public searchFields:SearchFields = new SearchFields();

  public fieldIds:  string[] = this.searchFields.PROJECT_ADVANCED_FIELDS;
  public fieldIdsMap = this.searchFields.PROJECT_FIELDS;
  public rangeFields:string[][] = this.searchFields.PROJECT_RANGE_FIELDS;
  public selectedFields:AdvancedField[] =  [];
  properties:EnvProperties;

  public resourcesQuery = "(oaftype exact project)";
  public csvParams: string;
  public disableForms: boolean = false;
  public loadPaging: boolean = true;
  public oldTotalResults: number = 0;
  public pagingLimit: number = 0;
  public isPiwikEnabled;
  public refineFields: string[] = this.searchFields.PROJECT_REFINE_FIELDS;
  @ViewChild(NewSearchPageComponent) searchPage: NewSearchPageComponent;
  @Input() simpleView: boolean = true;
  @Input() simpleSearchLink: string = "";
  advancedSearchLink: string = "";
  @Input() hasPrefix: boolean = true;
  @Input() openaireLink: string;
  @Input() includeOnlyResultsAndFilter: boolean = false;
  @Output() searchPageUpdates = new EventEmitter();
  constructor (private route: ActivatedRoute, private _searchProjectsService: SearchProjectsService ) {

    this.results =[];
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;




  }
  ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties= data.envSpecific;
        if (!this.simpleSearchLink) {
          this.simpleSearchLink = this.properties.searchLinkToProjects;
        }
        this.advancedSearchLink = this.properties.searchLinkToAdvancedProjects;
        this.searchUtils.baseUrl = (this.simpleView)?this.simpleSearchLink:this.advancedSearchLink;
         this.pagingLimit = data.envSpecific.pagingLimit;
         this.isPiwikEnabled = data.envSpecific.enablePiwikTrack;

      });
    //var errorCodes:ErrorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;
    var firstLoad = true;

    this.sub =  this.route.queryParams.subscribe(params => {
      this.loadPaging = true;
      if(params['page'] && this.searchUtils.page != params['page']) {
        this.loadPaging = false;
        this.oldTotalResults = this.searchUtils.totalResults;
      }
      var refine = true;
      if (this.searchUtils.page != ((params['page'] === undefined) ? 1 : +params['page']) && this.filters && !firstLoad) {
        refine = false;

      }
      let page = (params['page']=== undefined)?1:+params['page'];
      this.searchUtils.page = ( page <= 0 ) ? 1 : page;

      this.searchUtils.size = (params['size']=== undefined)?10:+params['size'];
      if(this.searchUtils.size != 5 && this.searchUtils.size != 10 && this.searchUtils.size != 20 && this.searchUtils.size != 50) {
        this.searchUtils.size = 10;
      }

      this.searchPage.fieldIds = this.fieldIds;
      this.selectedFields = [];
      this.searchPage.prepareSearchPage(this.fieldIds, this.selectedFields, this.refineFields, this.rangeFields, this.fieldIdsMap,this.customFilter,params, "project");
      this.getResults(this.searchPage.getSearchAPIQueryForAdvancedSearhFields(),  this.searchUtils.page, this.searchUtils.size, refine, this.searchPage.getSearchAPIQueryForRangeFields(params)+this.searchPage.getSearchAPIQueryForRefineFields(params, firstLoad));
      firstLoad = false;
    });
  }
  ngOnDestroy() {
    this.sub.unsubscribe();
  }
  sub: any;
  public getResults(parameters:string, page: number, size: number, refine: boolean, refineFieldsFilterQuery = null){
    if(page > this.pagingLimit) {
      size=0;
    }
    if(page <= this.pagingLimit || this.searchUtils.status == this.errorCodes.LOADING) {
      // if(parameters!= null && parameters != ''  ) {
      //   this.csvParams ="&fq=( "+this.resourcesQuery + "and (" + parameters + "))";
      // }else{
      //   this.csvParams ="&fq="+this.resourcesQuery;
      // }
      // this.csvParams += (refineFieldsFilterQuery?refineFieldsFilterQuery:'');
      //
      // //var errorCodes:ErrorCodes = new ErrorCodes();

      this.csvParams = (parameters ? ("&fq=("+parameters) : "") + (parameters ? ")" : "");
      this.csvParams += (refineFieldsFilterQuery ? refineFieldsFilterQuery : "");

      this.searchUtils.status = this.errorCodes.LOADING;
      //this.searchPage.openLoading();
      this.disableForms = true;
      this.searchPageUpdates.emit({disableForms: this.disableForms,searchUtils: this.searchUtils});
      this.results = [];
      this.searchUtils.totalResults = 0;

      //console.info("Advanced Search for Publications: Execute search query "+parameters);
       this._searchProjectsService.advancedSearchProjects(parameters,  page, size, this.properties,(refine) ? this.searchPage.getRefineFieldsQuery() : null, this.searchPage.getFields(), refineFieldsFilterQuery).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];
              this.results = data[1];
              // this.searchPage.updateBaseUrlWithParameters();
              if (refine) {
                this.filters = this.searchPage.prepareFiltersToShow(data[2], this.searchUtils.totalResults);
                this.rangeFilters = this.searchPage.prepareRangeFiltersToShow();
              }else{
                this.searchPage.buildPageURLParameters(this.filters, this.rangeFilters, false);
              }

            //var errorCodes:ErrorCodes = new ErrorCodes();
              this.searchUtils.status = this.errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = this.errorCodes.NONE;
              }
              //this.searchPage.closeLoading();
              this.disableForms = false;
            this.searchPageUpdates.emit({disableForms: this.disableForms, searchUtils: this.searchUtils})

              if(this.searchUtils.status == this.errorCodes.DONE) {
                // Page out of limit!!!
                let totalPages:any = this.searchUtils.totalResults/(this.searchUtils.size);
                if(!(Number.isInteger(totalPages))) {
                    totalPages = (parseInt(totalPages, 10) + 1);
                }
                if(totalPages < page) {
                  this.searchUtils.totalResults = 0;
                  this.searchUtils.status = this.errorCodes.OUT_OF_BOUND;
                }
              }
          },
          err => {
              //console.log(err);
              this.handleError("Error getting projects", err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

              //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = errorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.NOT_AVAILABLE;
              /*if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/

              //this.searchPage.closeLoading();
              this.disableForms = false;
            this.searchPageUpdates.emit({disableForms: this.disableForms, searchUtils: this.searchUtils})

          }
      );
    }
  }

  private handleError(message: string, error) {
    console.error("Projects advanced Search Page: "+message, error);
  }
}
