import {Component, EventEmitter, Input, Output} from '@angular/core';
import {FormBuilder} from "@angular/forms";
import {EnvProperties} from "../../utils/properties/env-properties";
import {SearchCustomFilter} from "./searchUtils.class";
import {ConfigurationService} from "../../utils/configuration/configuration.service";
import {Router} from "@angular/router";

@Component({
  selector: 'entities-selection',
  template: `
     <span  class="entitiesSelection portal-box uk-text-small " style="" >
      <mat-select *ngIf="show>1 && selectedEntity " [(value)]="selectedEntity"
                 (valueChange)="entityChanged()" [disableOptionCentering]="true" panelClass="matSelectionPanel">
        <mat-option
          *ngIf="simpleView && (show)>1 && !(this.customFilter && this.customFilter.queryFieldName=='communityId' )"
          value="all">All content
        </mat-option>
        <mat-option *ngIf="showResearchOutcomes" value="result">Research outcomes</mat-option>
        <mat-option *ngIf="showProjects" value="project">Projects</mat-option>
        <mat-option *ngIf="showDataProviders" value="dataprovider">Content providers</mat-option>
        <mat-option *ngIf="showOrganizations" value="organization">Organizations</mat-option>
      </mat-select>
      <div *ngIf="show==1 && currentEntity" >
        <div >
          <span *ngIf="currentEntity=='all'">All content</span>
          <span *ngIf="currentEntity=='result'">Research outcomes</span>
          <span *ngIf="currentEntity=='project'">Projects</span>
          <span *ngIf="currentEntity=='dataprovider'">Content providers</span>
          <span *ngIf="currentEntity=='organization'">Organizations</span>
        </div>

      </div>
    </span>




  `
})

export class EntitiesSelectionComponent {

  showResearchOutcomes: boolean = false;
  showProjects: boolean = false;
  showDataProviders: boolean = false;
  showOrganizations: boolean = false;
  @Input() properties: EnvProperties;
  @Input() customFilter: SearchCustomFilter = null;
  @Input() @Output() selectedEntity = "Research Outcomes";
  @Input() currentEntity = "Research Outcomes";
  @Input() simpleView: boolean = true;
  @Input() onChangeNavigate: boolean = true;
  @Output() selectionChange = new EventEmitter();
  show = 0;

  constructor(private _fb: FormBuilder, private config: ConfigurationService, private router: Router) {


  }


  ngOnInit() {
    this.show = 0;
    if (this.properties) {
      console.log(this.customFilter)
      this.config.getCommunityInformation(this.properties, (this.customFilter && this.customFilter.queryFieldName == "communityId") ? this.customFilter.valueId : this.properties.adminToolsCommunity).subscribe(data => {
        var showEntity = {};
        console.log(data)
        for (var i = 0; i < data['entities'].length; i++) {

          showEntity["" + data['entities'][i]["pid"] + ""] = data['entities'][i]["isEnabled"];
        }
        this.showResearchOutcomes = showEntity["publication"] || showEntity["dataset"] || showEntity["software"] || showEntity["orp"];
        this.showProjects = showEntity["project"];
        this.showOrganizations = showEntity["organization"];
        this.showDataProviders = showEntity["datasource"];

        if (this.customFilter && this.customFilter.queryFieldName == "communityId") {
          this.showOrganizations = false;
          if(!this.simpleView){
            this.showProjects = false;
            this.showDataProviders = false;
          }
        }
        if(this.showResearchOutcomes){
          this.show++;
        }
        if(this.showDataProviders){
          this.show++;
        }
        if(this.showOrganizations){
          this.show++;
        }
        if(this.showProjects){
          this.show++;
        }


      });
    }
    this.selectedEntity = this.currentEntity;
    this.selectionChange.emit({
      entity: this.selectedEntity,
      simpleUrl: this.getUrl(true),
      advancedUrl: this.getUrl(false)
    });
  }


  entityChanged() {

    if (!this.simpleView || this.onChangeNavigate) {
      this.router.navigate([this.getUrl(this.simpleView)]);
    } else {
      this.selectionChange.emit({
        entity: this.selectedEntity,
        simpleUrl: this.getUrl(true),
        advancedUrl: this.getUrl(false)
      });
    }

  }

  getUrl(simpleView: boolean) {
    let url = "";
    if (this.selectedEntity == "all") {
      url = (simpleView ? "/search/find/" : null);
    } else if (this.selectedEntity == "result") {
      url = (simpleView ? this.properties.searchLinkToResults : this.properties.searchLinkToAdvancedResults);
    } else if (this.selectedEntity == "project") {
      url = (simpleView ? this.properties.searchLinkToProjects : this.properties.searchLinkToAdvancedProjects);
    } else if (this.selectedEntity == "dataprovider") {
      url = (simpleView ? this.properties.searchLinkToDataProviders : this.properties.searchLinkToAdvancedDataProviders);
    } else if (this.selectedEntity == "organization") {
      url = (simpleView ? this.properties.searchLinkToOrganizations : this.properties.searchLinkToAdvancedOrganizations);
    }
    return url;
  }


}
